@@ -1,4 +1,5 @@
 #include "hreg.h"
+#include "hrpriv.h"
 #include <perl.h>
 
 
@@ -57,13 +58,10 @@ static inline void action_sanitize_str(HR_Action *action)
 
 static inline void action_sanitize_ptr(HR_Action *action)
 {
-    HR_DEBUG("Called!");
     if( (action->flags & HR_FLAG_SV_REFCNT_DEC) ) {
         HR_DEBUG("Decreasing reference count on SV=%p", action->key);
         SvREFCNT_dec((SV*)action->key);
         action->key = NULL;
-    } else {
-        HR_DEBUG("Flags=%d", action->flags);
     }
 }
 
@@ -76,10 +74,13 @@ static inline HR_Action* action_find_similar(
     HR_DEBUG("Request to find ktype=%d, kp=%p", ktype, hashref);
     HR_Action *cur = action_list;
     *lastp = cur;
-        
+    
+    int uhashref_is_opaque = (ktype & HR_KEY_SFLAG_HASHREF_OPAQUE);
+    ktype &= (~HR_KEY_SFLAG_HASHREF_OPAQUE);
+    
+    /*Prefilter for container comparison*/
     for(; cur; *lastp = cur, cur = cur->next) {
-        if(action_container_is_sv(cur)) {
-            
+        if(uhashref_is_opaque == 0 && action_container_is_sv(cur)) {
             if(action_container_is_rv(cur)) {
                 if(!cmp_container_RV2RV(cur->hashref, hashref)) {
                     continue;
@@ -122,6 +123,7 @@ static inline HR_Action* action_find_similar(
                 break;
         }
     }
+    HR_DEBUG("Couldn't find match");
     return NULL;
 }
 
@@ -146,7 +148,8 @@ HR_add_action(HR_Action *action_list,
         return;
     }
     
-    Newxz(cur, 1, HR_Action);
+    
+    Newxz_Action(cur);
     HR_DEBUG("cur is now %p", cur);
     last->next = cur;
     
@@ -197,7 +200,7 @@ HR_free_action(HR_Action *action)
     HR_Action *ret = action->next;
     action_sanitize(action);
     HR_DEBUG("Free: %p", action);
-    Safefree(action);
+    Free_Action(action);
     return ret;
 }
 
@@ -226,14 +229,14 @@ HR_del_action(HR_Action *action_list, SV *hashref, void *key, HR_KeyType_t ktype
         assert(last == cur == action_list);
         action_sanitize(cur);
         Copy(nextp, last, 1, HR_Action);
-        Safefree(nextp);
+        Free_Action(nextp);
         return HR_ACTION_DELETED;
     } else {
         HR_DEBUG("Delete %p hashref=%p", cur, cur->hashref);
         action_sanitize(cur);
         
         last->next = cur->next;
-        Safefree(cur);
+        Free_Action(cur);
     }
     return HR_ACTION_DELETED;
 }
@@ -273,34 +276,11 @@ HR_trigger_and_free_actions(HR_Action *action_list, SV *object)
         last = action_list;
         action_list = action_list->next;
         HR_DEBUG("Free %p", last);
-        Safefree(last);
+        Free_Action(last);
     }
     HR_DEBUG("Done");
 }
 
-#define FAKE_REFCOUNT (1 << 16)
-static inline U32
-refcnt_ka_begin(SV *sv)
-{
-    U32 ret = SvREFCNT(sv);
-    SvREFCNT(sv) = FAKE_REFCOUNT;
-    return ret;
-}
-
-static inline void
-refcnt_ka_end(SV *sv, U32 old_refcount)
-{
-    I32 effective_refcount = old_refcount + (SvREFCNT(sv) - FAKE_REFCOUNT);
-    if(effective_refcount <= 0 && old_refcount > 0) {
-        SvREFCNT(sv) = 1;
-        SvREFCNT_dec(sv);
-    } else {
-        SvREFCNT(sv) = effective_refcount;
-        if(effective_refcount != SvREFCNT(sv)) {
-            die("Detected negative refcount!");
-        }
-    }
-}
 
 static inline void
 invoke_coderef(SV *coderef, SV *object, char *key)
@@ -373,6 +353,11 @@ trigger_and_free_action(HR_Action *action_list, SV *object)
                 case HR_ACTION_TYPE_DEL_HV:
                 case HR_ACTION_TYPE_DEL_AV: {
                     
+                    if(SvREADONLY(container)) {
+                        warn("Container is read-only. Skipping deletion");
+                        goto GT_ACTION_FREE;
+                        break;
+                    }
                     /*Since this function can recurse, we need to ensure our
                      collection remains valid*/
                     HR_DEBUG("(KEEPALIVE): Refcount for container=%p is now %d", container, SvREFCNT(container));
@@ -419,7 +404,7 @@ trigger_and_free_action(HR_Action *action_list, SV *object)
                 
                 case HR_ACTION_TYPE_CALL_CFUNC: {
                     HR_DEBUG("Calling C Function!");
-                    ((HR_ActionCallback)(action_list->hashref)) (object, action_list->key);
+                    ((HR_ActionCallback)(action_list->hashref)) (object, action_list->key, action_list);
                     break;
                 }
                 
