@@ -3,7 +3,7 @@ use strict;
 use warnings;
 use base qw(Exporter);
 use Carp qw(carp confess);
-our @EXPORT;
+our (@EXPORT,%EXPORT_TAGS,@EXPORT_OK);
 
 my @logfuncs;
 
@@ -22,6 +22,7 @@ use Constant::Generate [qw(
     HR_KFLD_STRSCALAR
     HR_KFLD_REFSCALAR
     HR_KFLD_TABLEREF
+    HR_KFLD_ATTRHASH
     HR_KFLD_AVAILABLE
 )], export => 1;
 
@@ -30,6 +31,13 @@ use Constant::Generate [qw(
     HR_REVERSE_ATTRS
 )], export => 1;
 
+use Constant::Generate [qw(
+    DUPIDX_RLOOKUP
+    DUPIDX_FLOOKUP
+    DUPIDX_ALOOKUP
+    DUPIDX_SLOOKUP
+)], -tag => 'pp_constants' => -export_ok => 1;
+
 BEGIN {
     if(!$Module::Stubber::Status{'Log::Fu'}) {
         push @EXPORT, @logfuncs;
